<?php

namespace seraph_pds;

require_once( ABSPATH . 'wp-includes/pluggable.php' );

function OnAdminApi_UpdatePostInfo( $args )
{
	Wp::RemoveLangAttachmentFilters();

	$resCheckAccess = _AdminApi_CheckRights();
	if( $resCheckAccess )
		return( $resCheckAccess );

	$data = file_get_contents( 'php://input' );
	if( empty( $data ) )
		return( array( 'status' => 'invaliddata' ) );

	$data = json_decode( $data, true );
	if( !$data )
		return( array( 'status' => 'fail', 'status_descr' => 'Data JSON parse: ' . json_last_error_msg() ) );

	$post = null;
	$postId = 0;
	{
		if( isset( $args[ 'id' ] ) )
		{
			$postId = intval( $args[ 'id' ] );
			$post = get_post( $postId );
			if( !$post )
				return( array( 'status' => 'notfound' ) );

			if( $post -> post_type != $args[ 'type' ] )
				return( array( 'status' => 'wrongtype' ) );
		}
		else
		{
			if( empty( $data[ 'title' ] ) )
				$data[ 'title' ] = 'New ' . $args[ 'type' ];

			$postId = wp_insert_post( array( 'post_title' => wp_slash( $data[ 'title' ] ), 'post_type' => $args[ 'type' ], 'post_content' => '', 'comment_status' => 'open' ) );
			if( !( $postId > 0 ) )
				return( array( 'status' => 'fail' ) );

			unset( $data[ 'title' ] );

			$post = get_post( $postId );
		}
	}

	$arrWarn = array();
	$res = SetPostData( $post, $data, $arrWarn );

	return( array( 'status' => $res[ 'res' ], 'status_descr' => @$res[ 'descr' ], 'warnings' => $arrWarn, 'data' => array( 'id' => $postId, 'mediaUrl' => Wp::GetMediaUploadUrl( $post, true ), 'postUrl' => get_permalink( $post ), 'postSlug' => get_post_field( 'post_name', $post ), 'postTitle' => get_post_field( 'post_title', $post ) ) ) );
}

function OnAdminApi_UpdatePostData( $args )
{
	Wp::RemoveLangAttachmentFilters();

	$postId = intval( $args[ 'id' ] );
	$post = get_post( $postId );
	if( empty( $post ) )
		return( array( 'status' => 'notfound' ) );

	$data = file_get_contents( 'php://input' );
	if( empty( $data ) )
		return( array( 'status' => 'invaliddata' ) );

	$data = json_decode( $data, true );
	if( !$data )
		return( array( 'status' => 'fail', 'status_descr' => 'Data JSON parse: ' . json_last_error_msg() ) );

	$arrWarn = array();
	$res = SetPostData( $post, $data, $arrWarn );

	return( array( 'status' => $res[ 'res' ], 'status_descr' => @$res[ 'descr' ], 'warnings' => $arrWarn ) );
}

function OnAdminApi_SanitizeLabel( $args )
{
	return( sanitize_title( $args[ "str" ] ) );
}

function OnAdminApi_GetAttachmentIdFromUrl( $args )
{
	return( Wp::GetAttachmentIdFromUrl( $args[ 'url' ], @$args[ 'lang' ] ) );
}

function OnAdminApi_GetUrl( $args )
{
	return( getUrl( $args[ 'url' ], !empty( $args[ 'body' ] ) ) );
}

function OnAdminApi_UploadImage( $args )
{
	Wp::RemoveLangAttachmentFilters();

	$data = fopen( 'php://input', 'rb' );

	$res = UploadImage( $args[ 'filename' ], $args[ 'contentType' ], $data, @$args[ 'postId' ], @$args[ 'addToAttachments' ] == "true", @$args[ 'overwrite' ] == "true", @$args[ 'uploadDir' ], @$args[ 'uploadDirSubDir' ], array( 'title' => @$args[ 'title' ], 'altText' => @$args[ 'altText' ], 'description' => @$args[ 'description' ], 'caption' => @$args[ 'caption' ] ), @$args[ 'lang' ] );

	$ret = null;
	if( !is_wp_error( $res ) )
		$ret = array( 'status' => 'ok', 'data' => $res );
	else
		$ret = array( 'status' => $res -> get_error_code(), 'status_descr' => $res -> get_error_message() );

	fclose( $data );
	return( $ret );
}

function OnAdminApi_UpdatePostTypeTaxonomies( $args )
{
	$data = file_get_contents( 'php://input' );
	if( empty( $data ) )
		return( null );

	$data = json_decode( $data, true );
	if( !$data )
		return( null );

	$lang = @$args[ 'lang' ];

	$res = Wp::UpdatePostTypeTaxonomies( $data, $args[ 'taxonomy' ], $args[ 'postType' ], $lang );
	return( $res );
}

function _AdminApi_CheckRights()
{
	if( current_user_can( 'publish_posts' ) )
		return( null );
	return( array( 'status' => 'accessdenied' ) );
}

function SetPostData( $post, $data, &$arrWarn )
{
	$availablePlugins = Plugin::GetAvailablePlugins();

	{
		$argsUpdatePost = array();

		{
			$title = @$data[ 'title' ];
			if( $title )
				$argsUpdatePost[ 'post_title' ] = $title;
		}

		{
			$slug = @$data[ 'slug' ];
			if( $slug !== null )
				$argsUpdatePost[ 'post_name' ] = $slug;
		}

		{
			$excerpt = @$data[ 'excerpt' ];
			if( $excerpt !== null )
				$argsUpdatePost[ 'post_excerpt' ] = $excerpt;
		}

		{
			$text = @$data[ 'text' ];
			if( $text !== null )
				$argsUpdatePost[ 'post_content' ] = $text;
		}

		if( !empty( $argsUpdatePost ) )
		{
			$argsUpdatePost[ 'ID' ] = $post -> ID;

			$updRes = wp_update_post( wp_slash( $argsUpdatePost ), true );
			if( is_wp_error( $updRes ) )
				return( array( 'res' => 'fail', 'descr' => $updRes -> get_error_message() ) );
		}
	}

	{
		$categories = @$data[ 'categories' ];
		if( $categories !== null )
		{
			$applied = false;

			$postTypesTaxonomy = Wp::GetPostsTaxonomies( Wp::POST_TAXONOMY_TYPE_CATEGORY );
			if( !empty( $postTypesTaxonomy ) )
			{
				$postTaxonomy = $postTypesTaxonomy[ $post -> post_type ];
				if( !empty( $postTaxonomy ) )
				{

					wp_set_post_terms( $post -> ID, $categories, $postTaxonomy );

					$applied = true;
				}
			}

			if( !$applied )
				$arrWarn[] = _x( 'CatsNotSet', 'admin.Msg', 'seraphinite-post-docx-source' );
		}
	}

	{
		$keywords = @$data[ 'keywords' ];
		if( $keywords !== null )
		{
			$applied = false;

			$postTypesTaxonomy = Wp::GetPostsTaxonomies( Wp::POST_TAXONOMY_TYPE_TAG );
			if( !empty( $postTypesTaxonomy ) )
			{
				$postTaxonomy = $postTypesTaxonomy[ $post -> post_type ];
				if( !empty( $postTaxonomy ) )
				{

					wp_set_post_terms( $post -> ID, $keywords, $postTaxonomy );

					$applied = true;
				}
			}

			if( !$applied )
				$arrWarn[] = _x( 'KeywordsNotSet', 'admin.Msg', 'seraphinite-post-docx-source' );
		}
	}

	{
		$featuredImage = @$data[ 'featuredImage' ];
		if( $featuredImage !== null )
		{
			if( !$featuredImage )
			{
				$thumbnailId = get_post_thumbnail_id( $post );
				if( $thumbnailId )
				{
					delete_post_thumbnail( $post );

					$updRes = wp_update_post( array( 'ID' => $thumbnailId, 'post_parent' => 0 ), true );
					if( is_wp_error( $updRes ) )
						$arrWarn[] = sprintf( _x( 'FeaturImgNotUpdated_%1$s', 'admin.Msg', 'seraphinite-post-docx-source' ), $updRes -> get_error_message() );
				}
			}
			else
			{

				set_post_thumbnail( $post, $featuredImage );

				$updRes = wp_update_post( array( 'ID' => $featuredImage, 'post_parent' => $post -> ID ), true );
				if( is_wp_error( $updRes ) )
					$arrWarn[] = sprintf( _x( 'FeaturImgNotSet_%1$s', 'admin.Msg', 'seraphinite-post-docx-source' ), $updRes -> get_error_message() );
			}
		}
	}

	{
		$images = @$data[ 'wooProductGalleryImages' ];
		if( $images !== null )
			update_post_meta( $post -> ID, '_product_image_gallery', implode( ',', $images ) );
	}

	{
		$titleSeo = @$data[ 'titleSeo' ];
		if( $titleSeo !== null )
		{
			$bSet = false;

			if( in_array( 'all-in-one-seo-pack', $availablePlugins ) )
			{
				if( empty( $titleSeo ) )
					delete_post_meta( $post -> ID, '_aioseop_title' );
				else
					update_post_meta( $post -> ID, '_aioseop_title', $titleSeo );
				$bSet = true;
			}

			if( in_array( 'wordpress-seo', $availablePlugins ) )
			{
				if( empty( $titleSeo ) )
					delete_post_meta( $post -> ID, '_yoast_wpseo_title' );
				else
					update_post_meta( $post -> ID, '_yoast_wpseo_title', $titleSeo );
				$bSet = true;
			}

			if( !$bSet )
				$arrWarn[] = _x( 'TitleSeoNotSet', 'admin.Msg', 'seraphinite-post-docx-source' );
		}
	}

	{
		$descrSeo = @$data[ 'descrSeo' ];
		if( $descrSeo !== null )
		{
			$bSet = false;

			if( in_array( 'all-in-one-seo-pack', $availablePlugins ) )
			{
				if( empty( $descrSeo ) )
					delete_post_meta( $post -> ID, '_aioseop_description' );
				else
					update_post_meta( $post -> ID, '_aioseop_description', $descrSeo );
				$bSet = true;
			}

			if( in_array( 'wordpress-seo', $availablePlugins ) )
			{
				if( empty( $descrSeo ) )
					delete_post_meta( $post -> ID, '_yoast_wpseo_metadesc' );
				else
					update_post_meta( $post -> ID, '_yoast_wpseo_metadesc', $descrSeo );
				$bSet = true;
			}

			if( !$bSet )
				$arrWarn[] = _x( 'DescrSeoNotSet', 'admin.Msg', 'seraphinite-post-docx-source' );
		}
	}

	{
		$lang = @$data[ 'lang' ];
		if( $lang !== null )
		{
			$langsAvail = Wp::GetLangs();
			if( isset( $langsAvail[ $lang ] ) )
			{
				$hr = Wp::SetPostLang( $post -> ID, $lang );
				if( Gen::HrFail( $hr ) && $hr != Gen::E_NOTIMPL )
					$arrWarn[] = sprintf( _x( 'LangNotSet_%1$s%2$x', 'admin.Msg', 'seraphinite-post-docx-source' ), $lang, $hr );
			}
			else
				$arrWarn[] = sprintf( _x( 'LangNotAvail_%1$s', 'admin.Msg', 'seraphinite-post-docx-source' ), $lang );
		}
	}

	{
		$attrs = @$data[ 'blocks' ];
		if( $attrs )
		{
			foreach( $attrs as $attrKey => $attr )
			{
				$attr = wp_slash( $attr );

				$posSep = strpos( $attrKey, '.' );
				if( $posSep !== false )
				{
					$arrFldPath = substr( $attrKey, $posSep + 1 );
					$attrKey = substr( $attrKey, 0, $posSep );

					$attrArr = get_post_meta( $post -> ID, $attrKey, true );
					if( !is_array( $attrArr ) )
						$attrArr = array();
					Gen::SetArrField( $attrArr, $arrFldPath, $attr );
					update_post_meta( $post -> ID, $attrKey, $attrArr );
				}
				else
				{
					if( empty( $attr ) )
						delete_post_meta( $post -> ID, $attrKey );
					else
						update_post_meta( $post -> ID, $attrKey, $attr );
				}
			}
		}
	}

	return( array( 'res' => 'ok' ) );
}

function UploadImage( $fileName, $contentType, $data, $postId = 0, $addToAttachments = true, $overwrite = false, $uploadDir = null, $uploadDirSubDir = null, $attrs = null, $lang = null )
{
	global $ctxUploadImage;

	$ctxUploadImage = array(
		'addToAttachments'		=> $addToAttachments,
		'overwrite'				=> $overwrite,
		'fileName'				=> $fileName,

		'uploadDirSubDir'		=> $uploadDirSubDir,
		'lang'					=> $lang
	);

	if( !current_user_can( 'upload_files' ) )
		return( new \WP_Error( 'access_denied', _x( 'UserNotUploadFiles', 'admin.Msg', 'seraphinite-post-docx-source' ) ) );

	if( $postId && !current_user_can( 'edit_post', $postId ) )
		return( new \WP_Error( 'access_denied', _x( 'UserNotEditPost', 'admin.Msg', 'seraphinite-post-docx-source' ) ) );

	global $post, $post_id, $_GET;

	$post_prev = $post;
	$post_id_prev = $post_id;

	if( $postId )
	{
		$post = null;
		$post_id = $postId;
	}

	$fileNamePathTmp = Wp::GetTempFile();

	add_filter( 'wp_unique_filename',			'seraph_pds\\_wp_unique_filename_UploadImage', 1, 4 );
	add_filter( 'pre_move_uploaded_file',		'seraph_pds\\_pre_move_uploaded_file_UploadImage', 1, 4 );
	add_filter( 'wp_handle_upload',				'seraph_pds\\_handle_upload_UploadImage', 99999, 2 );
	add_filter( 'upload_dir',					'seraph_pds\\_upload_dir_UploadImage' );

	$res = _UploadImage( $fileNamePathTmp, $fileName, $contentType, $data, $postId, $addToAttachments, $overwrite, $attrs, $lang );

	remove_filter( 'upload_dir',				'seraph_pds\\_upload_dir_UploadImage' );
	remove_filter( 'wp_handle_upload',			'seraph_pds\\_handle_upload_UploadImage', 99999 );
	remove_filter( 'pre_move_uploaded_file',	'seraph_pds\\_pre_move_uploaded_file_UploadImage', 1 );
	remove_filter( 'wp_unique_filename',		'seraph_pds\\_wp_unique_filename_UploadImage', 1, 4 );

	@unlink( $fileNamePathTmp );

	if( $postId )
	{
		$post = $post_prev;
		$post_id = $post_id_prev;
	}

	return( $res );
}

function _wp_unique_filename_UploadImage( $filename, $ext, $dir, $unique_filename_callback )
{
	global $ctxUploadImage;

	if( $ctxUploadImage[ 'overwrite' ] )
		$filename = $ctxUploadImage[ 'fileName' ];
	else
		Fs::CreateEmptyFile( $dir . '/' . $filename );

	return( $filename );
}

function _pre_move_uploaded_file_UploadImage( $move_new_file, $file, $new_file, $type )
{
	global $ctxUploadImage;

	if( @rename( $file[ 'tmp_name' ], $new_file ) === false )
	{
		$ctxUploadImage[ 'error' ] = 'Can\'t write to "' . $new_file . '"';
		return( $move_new_file );
	}

	return( true );
}

function _handle_upload_UploadImage( $info, $action )
{
	global $ctxUploadImage;

	if( $ctxUploadImage[ 'overwrite' ] )
	{
		$newFileExt = Gen::GetFileExt( $info[ 'file' ] );
		if( Gen::GetFileExt( $ctxUploadImage[ 'fileName' ] ) != $newFileExt )
		{
			$oldFileName = Gen::GetFileName( $ctxUploadImage[ 'fileName' ], true );

			$correctedNewFilePathName = dirname( $info[ 'file' ] ) . '/' . $oldFileName . '.' . $newFileExt;
			if( @rename( $info[ 'file' ], $correctedNewFilePathName ) === false )
			{
				$ctxUploadImage[ 'error' ] = 'Can\'t write to "' . $correctedNewFilePathName . '"';
			}
			else
			{
				$info[ 'file' ] = $correctedNewFilePathName;
				$info[ 'url' ] = dirname( $info[ 'url' ] ) . '/' . $oldFileName . '.' . $newFileExt;
			}
		}
	}

	$ctxUploadImage[ 'url' ] = $info[ 'url' ];
	if( $ctxUploadImage[ 'addToAttachments' ] && $ctxUploadImage[ 'overwrite' ] )
		$ctxUploadImage[ 'prevAttachmentId' ] = Wp::GetAttachmentIdFromUrl( $info[ 'url' ], $ctxUploadImage[ 'lang' ] );

	if( !$ctxUploadImage[ 'addToAttachments' ] )
		$info[ 'error' ] = 'warnAttachmentWasNotPut';

	return( $info );
}

function _upload_dir_UploadImage( $uploads )
{
	global $ctxUploadImage;

	$uploadDirSubDir = @$ctxUploadImage[ 'uploadDirSubDir' ];
	if( $uploadDirSubDir !== null )
	{

		{
			$subDirLen = strlen( $uploads[ 'subdir' ] );
			if( $subDirLen )
			{
				$uploads[ 'subdir' ] = '';
				$uploads[ 'path' ] = substr( $uploads[ 'path' ], 0, strlen( $uploads[ 'path' ] ) - $subDirLen );
				$uploads[ 'url' ] = substr( $uploads[ 'url' ], 0, strlen( $uploads[ 'url' ] ) - $subDirLen );
			}
		}

		{
			$uploads[ 'subdir' ] .= $uploadDirSubDir;
			$uploads[ 'path' ] .= $uploadDirSubDir;
			$uploads[ 'url' ] .= $uploadDirSubDir;
		}
	}

	return( $uploads );
}

function _UploadImage( $fileNamePathTmp, $fileName, $contentType, $data, $postId, $addToAttachments, $overwrite, $attrs, $lang )
{
	global $ctxUploadImage;

	$fileNamePathTmpSize = 0;
	{
		$file = @fopen( $fileNamePathTmp, 'wb' );
		if( $file === false )
			return( new \WP_Error( 'internal_error', sprintf( _x( 'NotCreateTmpFile_%1$s', 'admin.Msg', 'seraphinite-post-docx-source' ), $fileNamePathTmp ) ) );

		if( Fs::StreamCopy( $data, $file ) != Gen::S_OK )
			$err = new \WP_Error( 'internal_error', sprintf( _x( 'NotCopyToTmpFile_%1$s', 'admin.Msg', 'seraphinite-post-docx-source' ), $fileNamePathTmp ) );

		$fileNamePathTmpSize = @filesize( $fileNamePathTmp );

		fclose( $file );
		if( $err )
			return( $err );
	}

	if( !$fileNamePathTmpSize )
		return( new \WP_Error( 'error_void_content', _x( 'NotUploadZeroContent', 'admin.Msg', 'seraphinite-post-docx-source' ) ) );

	$_FILES[ 'f' ] = array(
		'name'					=> $fileName,
		'tmp_name'				=> $fileNamePathTmp,
		'type'					=> $contentType,
		'size'					=> $fileNamePathTmpSize,
		'error'					=> 0,
	);

	$_FILES[ 'f' ] = apply_filters( 'wp_handle_upload_prefilter', $_FILES[ 'f' ] );

	$updateAttachment = $addToAttachments && $overwrite;

	$attachmentId = null;
	$metadata = null;
	{
		$res = media_handle_upload( 'f', $postId, array(), array( 'test_form' => false, 'action' => 'wp_handle_sideload' ) );
		if( is_wp_error( $res ) )
		{
			$warnAttachmentWasNotPut = $res -> get_error_code() == 'upload_error' && $res -> get_error_message() == 'warnAttachmentWasNotPut';
			if( !$warnAttachmentWasNotPut )
				return( $res );
		}
		else
		{
			if( !empty( @$ctxUploadImage[ 'error' ] ) )
				return( new \WP_Error( 'internal_error', @$ctxUploadImage[ 'error' ] ) );

			$attachmentId = $res;

			if( $updateAttachment )
			{
				$prevAttachmentId = $ctxUploadImage[ 'prevAttachmentId' ];

				if( $prevAttachmentId )
				{
					$newAttachment = get_post( $attachmentId, ARRAY_A );
					$newAttachmentMetadata = wp_get_attachment_metadata( $attachmentId );

					{
						update_attached_file( $newAttachment[ 'ID' ], '' );

						$res = wp_delete_attachment( $newAttachment[ 'ID' ], true );
						if( is_wp_error( $res ) )
							return( $res );
					}

					$newAttachment[ 'ID' ] = $prevAttachmentId;

					{
						$res = wp_update_post( $newAttachment, true );
						if( is_wp_error( $res ) )
							return( $res );
					}

					wp_update_attachment_metadata( $prevAttachmentId, $newAttachmentMetadata );

					$attachmentId = $prevAttachmentId;
					$metadata = $newAttachmentMetadata;
				}
				else
					$metadata = wp_get_attachment_metadata( $attachmentId );
			}
			else
				$metadata = wp_get_attachment_metadata( $attachmentId );

			if( $addToAttachments )
			{

				{
					if( $lang !== null )
					{
						$hr = Wp::SetPostLang( $attachmentId, $lang );
						if( Gen::HrFail( $hr ) && $hr != Gen::E_NOTIMPL )
							return( new \WP_Error( 'internal_error', 'Can\'t set \'' . $lang . '\' language: ' . sprintf( '0x%08X', $hr ) ) );
					}
				}

				{
					$updateData = array();

					$title = @$attrs[ 'title' ];
					if( $title )
						$updateData[ 'post_title' ] = $title;

					$altText = @$attrs[ 'altText' ];
					if( $altText )
						$updateData[ '_wp_attachment_image_alt' ] = $altText;

					$description = @$attrs[ 'description' ];
					if( $description )
						$updateData[ 'post_content' ] = $description;

					$caption = @$attrs[ 'caption' ];
					if( $caption )
						$updateData[ 'post_excerpt' ] = $caption;

					if( count( $updateData ) )
					{
						$res = Wp::UpdateAttachment( $attachmentId, wp_slash( $updateData ), true );
						if( is_wp_error( $res ) )
							return( $res );
					}
				}
			}

			if( !wp_prepare_attachment_for_js( $attachmentId ) )
				return( new \WP_Error( 'internal_error', 'prepare_attachment_for_js_failed' ) );
		}
	}

	return( array( 'url' => Net::Url2Uri( $ctxUploadImage[ 'url' ], true ), 'attachmentId' => $attachmentId, 'metadata' => $metadata ) );
}

